#include <iostream>
#include <cmath>

using namespace std;

//
// vector functions
//
inline double *
vector_init ( const unsigned  i )
{
    return new double[i];
}

void fill  ( const unsigned  n, const double f, double * y )
{
    for ( unsigned i = 0; i < n; i++ )
        y[i] = f;
}

void scale ( const unsigned  n, const double f, double * y )
{
    for ( unsigned i = 0; i < n; i++ )
        y[i] *= f;
}

void add   ( const unsigned   n,
             const double     f,
             const double *   x,
             double *         y )
{
    for ( unsigned i = 0; i < n; i++ )
        y[i] += f * x[i];
}

double
dot ( const unsigned  n, const double * x, const double * y )
{
    double  d = 0.0;

    for ( unsigned i = 0; i < n; i++ )
        d += x[i] * y[i];
    
    return d;
}

inline double
norm2 ( const unsigned  n, const double * x )
{
    return sqrt( dot( n, x, x ) );
}

//
// matrix functions
//
inline double *
matrix_init ( const unsigned  n, const unsigned  m )
{
    return new double[ n * m ];
}

double
get_entry ( const unsigned  n, const unsigned  m,
            const unsigned  i, const unsigned  j,
            const double * M )
{
    return M[ i*m + j ];
}

double
add_entry ( const unsigned  n, const unsigned  m,
            const unsigned  i, const unsigned  j,
            const double    f, double *        M )
{
    M[ i*m + j ] += f;
}

void
fill ( const unsigned  n, const unsigned  m,
       const double f, double * M )
{
    fill( n*m, f, M );
}

void
scale ( const unsigned  n,
        const unsigned  m,
        const double    f,
        double *        M )
{
    scale( n*m, f, M );
}

void
add ( const unsigned  n,
      const unsigned  m,
      const double    f,
      const double *  A,
      double *        M )
{
    add( n*m, f, A, M );
}

inline double
normF ( const unsigned  n,
        const unsigned  m,
        double *        M )
{
    return norm2( n*m, M );   // use vector based norm2
}

void
mul_vec ( const unsigned  n, const unsigned  m,
          const double alpha, const double * M, const double * x,
          double * y )
{
    for ( unsigned i = 0; i < n; i++ )
    {
        double  f = 0.0;

        for ( unsigned j = 0; j < m; j++ )
            f += get_entry( n, m, i, j, M ) * x[j];
        // alternative: f = dot( m, & M[ i * m ], x );

        y[i] += alpha * f;
    }
}

void
mul_mat ( const unsigned n, const unsigned m, const unsigned k,
          const double alpha, const double * A, const double * B,
          double * C )
{
    for ( unsigned i = 0; i < n; i++ )
        for ( unsigned j = 0; j < m; j++ )
        {
            double  f = 0.0;

            for ( unsigned l = 0; l < k; l++ )
                f += get_entry( n, k, i, l, A ) *
                     get_entry( k, m, l, j, B );

            add_entry( n, m, i, j, f, C );
        }
}

//
// main function
//
int
main ()
{
    double *  M = matrix_init( 10, 10 );
    double *  x = vector_init( 10 );
    double *  y = vector_init( 10 );

    fill( 10, 10, 2.0, M );
    fill( 10, 3.0, x );
    fill( 10, 0.0, y );

    mul_vec( 10, 10, 1.0, M, x, y );

    cout << dot( 10, x, y ) << endl;

    return 0;
}
