#include <cassert>

#include "vector.hh"

namespace LinAlg
{

//
// ctors
//

vector_t::vector_t ( const size_t  n )
{
    size   = n;
    coeffs = new real_t[n];
}

vector_t::vector_t ( const vector_t &  v )
{
    *this = v;
}


//
// dtor
//

vector_t::~vector_t ()
{
    delete[] coeffs;

    coeffs = NULL;
    size   = 0;
}

//
// BLAS
//

//
// set vector ≡ f
//
void
vector_t::fill  ( const real_t  f )
{
    for ( size_t  i = 0; i < size; i++ )
        coeffs[i] = f;
}

//
// set vector = vector * f
//
void
vector_t::scale ( const real_t  f )
{
    for ( size_t  i = 0; i < size; i++ )
        coeffs[i] *= f;
}

//
// set vector = vector + f * v
//
void
vector_t::add   ( const real_t      f,
                  const vector_t &  v )
{
    assert( v.size == size );

    for ( size_t  i = 0; i < size; i++ )
        coeffs[i] += f * v.coeffs[i];
}

//
// return <vector,v>
//
real_t
vector_t::dot ( const vector_t &  v ) const
{
    assert( v.size == size );

    real_t  d = 0.0;
    
    for ( size_t  i = 0; i < size; i++ )
        d += coeffs[i] * v.coeffs[i];

    return d;
}

}// namespace LinAlg
