#ifndef __VECTOR_HH
#define __VECTOR_HH

#include <cmath>

namespace LinAlg
{

//
// basic type definitions
//
typedef  unsigned int   uint;
typedef  double         real_t;

//
// vector type for linear algebra
//

struct vector_t
{
    size_t    size;
    real_t *  coeffs;

    //
    // ctors
    //
    
    vector_t ( const size_t      n = 0 );
    vector_t ( const vector_t &  v );

    //
    // dtor
    //
    
    ~vector_t ();

    //
    // BLAS
    //

    // set vector ≡ f
    void fill  ( const real_t      f );

    // set vector = vector * f
    void scale ( const real_t      f );

    // set vector = vector + f * v
    void add   ( const real_t      f,
                 const vector_t &  v );

    // return <vector,v>
    real_t dot ( const vector_t &  v ) const;

    // return ∥vector∥₂
    real_t norm2 () const
    {
        return std::sqrt( dot( *this ) );
    }
};

}// namespace LinAlg

#endif
